@@ -6,6 +6,8 @@ basic constants and utility functions
 import datetime as dt
 import os
 import time
+import logging
+import inspect
 from decimal import Decimal
 import requests
 from functools import wraps
@@ -23,6 +25,8 @@ from scipy import optimize
 
 from xalpha import __path__
 
+logger = logging.getLogger(__name__)
+
 # date obj of today
 today = lambda: dt.datetime.combine(dt.date.today(), dt.time.min)
 
@@ -154,18 +158,64 @@ def convert_date(date):
         return date
 
 
-def reconnect(tries=5):
+def next_onday(dtobj):
+    dtobj += dt.timedelta(1)
+    while dtobj.strftime("%Y-%m-%d") not in opendate:
+        dtobj += dt.timedelta(1)
+    return dtobj
+
+
+def last_onday(dtobj):
+    dtobj -= dt.timedelta(1)
+    while dtobj.strftime("%Y-%m-%d") not in opendate:
+        dtobj -= dt.timedelta(1)
+    return dtobj
+
+
+def scale_dict(d, scale=1, ulimit=100, dlimit=50, aim=None):
+    t = sum([v for _, v in d.items()])
+    if t * scale > ulimit:
+        scale = ulimit / t
+    elif t * scale < dlimit:
+        scale = dlimit / t
+    if aim:
+        scale = aim / t
+    for k, v in d.items():
+        d[k] = v * scale
+    return d
+
+
+def reconnect(tries=5, timeout=12):
     def robustify(f):
         @wraps(f)
         def wrapper(*args, **kws):
+            import xalpha.provider as xp
+
+            if getattr(xp, "proxy", None):
+                kws["proxies"] = {"http": xp.proxy, "https": xp.proxy}
+                kws["timeout"] = timeout
+                logger.debug("Using proxy %s" % xp.proxy)
+            if args:
+                url = args[0]
+            else:
+                url = kws.get("url", "")
+
             for count in range(tries):
                 try:
+                    logger.debug(
+                        "Fetching url: %s . Inside function `%s`"
+                        % (url, inspect.stack()[1].function)
+                    )
                     r = f(*args, **kws)
                     return r
                 except connection_errors as e:
+                    logger.warning("Fails at fetching url: %s. Try again." % url)
                     if count == tries - 1:
-                        print(*args, sep="\n")
-                        print("still wrong after several tries")
+                        logger.critical(
+                            "Still wrong at fetching url: %s. after %s tries."
+                            % (url, tries)
+                        )
+                        logger.error("Fails due to %s" % e.args[0])
                         raise e
                     time.sleep(0.5 * count)
 
@@ -244,3 +294,23 @@ def rpost_json(*args, **kws):
 #                 print("still wrong after several tries")
 #                 raise e
 #             time.sleep(0.5*count)
+
+
+## simple subsitution for holdings.py
+
+holdings = {}
+holdings["501018"] = {
+    "etfs/etfs-brent-1mth-uk": 17.51,
+    "etfs/etfs-brent-crude": 15.04,
+    "etfs/etfs-crude-oil": 7.34,
+    "etfs/ipath-series-b-sp-gsci-crd-oil-tr": 0.06,
+    "etfs/powershares-db-oil-fund": 11.6,
+    "etfs/ubs-cmci-oil-sf-usd": 8.68,
+    "etfs/united-states-12-month-oil": 8.14,
+    "etfs/united-states-brent-oil-fund-lp": 15.42,
+    "etfs/united-states-oil-fund": 9.63,
+}
+holdings["501018rt"] = {
+    "commodities/brent-oil": 40 * 0.9,
+    "commodities/crude-oil": 60 * 0.9,
+}
