@@ -9,8 +9,10 @@ import sys
 import datetime as dt
 import numpy as np
 import pandas as pd
+import logging
 from bs4 import BeautifulSoup
-from functools import wraps
+from functools import wraps, lru_cache
+from uuid import uuid4
 from sqlalchemy import exc
 from dateutil.relativedelta import relativedelta
 
@@ -23,12 +25,13 @@ try:
         get_query_count,
         finance,
         get_index_stocks,
+        macro,
     )
 
     # 本地导入
 except ImportError:
     try:
-        from jqdata import finance  # 云平台导入
+        from jqdata import finance, macro  # 云平台导入
     except ImportError:
         pass
 
@@ -40,6 +43,7 @@ from xalpha.exceptions import DataPossiblyWrong, ParserFailure
 
 thismodule = sys.modules[__name__]
 xamodule = sys.modules["xalpha"]
+logger = logging.getLogger(__name__)
 
 
 def today_obj():
@@ -98,26 +102,49 @@ def get_historical_fromxq(code, count):
     return df
 
 
-def get_cninvesting(curr_id, st_date, end_date):
-    r = rpost(
-        "https://cn.investing.com/instruments/HistoricalDataAjax",
-        data={
-            "curr_id": curr_id,
-            #  "smlID": smlID,  # ? but seems to be fixed with curr_id, it turns out it doesn't matter
-            "st_date": st_date,  # %Y/%m/%d
-            "end_date": end_date,
-            "interval_sec": "Daily",
-            "sort_col": "date",
-            "sort_ord": "DESC",
-            "action": "historical_data",
-        },
-        headers={
-            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_4)\
-                AppleWebKit/537.36 (KHTML, like Gecko)",
-            "Host": "cn.investing.com",
-            "X-Requested-With": "XMLHttpRequest",
-        },
-    )
+def get_historical_fromcninvesting(curr_id, st_date, end_date, app=False):
+    data = {
+        "curr_id": curr_id,
+        #  "smlID": smlID,  # ? but seems to be fixed with curr_id, it turns out it doesn't matter
+        "st_date": st_date,  # %Y/%m/%d
+        "end_date": end_date,
+        "interval_sec": "Daily",
+        "sort_col": "date",
+        "sort_ord": "DESC",
+        "action": "historical_data",
+    }
+    if not app:  # fetch from web api
+        r = rpost(
+            "https://cn.investing.com/instruments/HistoricalDataAjax",
+            data=data,
+            headers={
+                "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_4)\
+                    AppleWebKit/537.36 (KHTML, like Gecko)",
+                "Host": "cn.investing.com",
+                "X-Requested-With": "XMLHttpRequest",
+            },
+        )
+    else:  # fetch from app api
+        r = rpost(
+            "https://cnappapi.investing.com/instruments/HistoricalDataAjax",
+            data=data,
+            headers={
+                "Accept": "*/*",
+                "Accept-Encoding": "gzip",
+                "Accept-Language": "zh-cn",
+                "Cache-Control": "no-cache",
+                "Connection": "keep-alive",
+                "User-Agent": "Investing.China/0.0.3 CFNetwork/1121.2.2 Darwin/19.3.0",
+                "ccode": "CN",
+                #'ccode_time': '1585551041.986028',
+                "x-app-ver": "117",
+                "x-meta-ver": "14",
+                "x-os": "ios",
+                "x-uuid": str(uuid4()),
+                "Host": "cn.investing.com",
+                "X-Requested-With": "XMLHttpRequest",
+            },
+        )
     s = BeautifulSoup(r.text, "lxml")
     dfdict = {}
     cols = []
@@ -165,17 +192,37 @@ def dstr2dobj(dstr):
     return d_obj
 
 
-def get_investing_id(suburl):
-    url = "https://cn.investing.com"
+@lru_cache(maxsize=512)
+def get_investing_id(suburl, app=False):
+    if not app:
+        url = "https://cn.investing.com"
+    else:
+        url = "https://cnappapi.investing.com"
     if not suburl.startswith("/"):
         url += "/"
     url += suburl
-    r = rget(
-        url,
-        headers={
+    if not app:
+        headers = {
             "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_4) AppleWebKit/537.36"
-        },
-    )
+        }
+    else:
+        headers = {
+            "Accept": "*/*",
+            "Accept-Encoding": "gzip",
+            "Accept-Language": "zh-cn",
+            "Cache-Control": "no-cache",
+            "Connection": "keep-alive",
+            "User-Agent": "Investing.China/0.0.3 CFNetwork/1121.2.2 Darwin/19.3.0",
+            "ccode": "CN",
+            #'ccode_time': '1585551041.986028',
+            "x-app-ver": "117",
+            "x-meta-ver": "14",
+            "x-os": "ios",
+            "x-uuid": str(uuid4()),
+            "Host": "cn.investing.com",
+            "X-Requested-With": "XMLHttpRequest",
+        }
+    r = rget(url, headers=headers,)
     s = BeautifulSoup(r.text, "lxml")
     pid = s.find("span", id="last_last")["class"][-1].split("-")[1]
     return pid
@@ -204,6 +251,27 @@ def get_rmb(start=None, end=None, prev=360, currency="USD/CNY"):
     :param currency:
     :return: pd.DataFrame
     """
+    bl = ["USD", "EUR", "100JPY", "HKD", "GBP", "AUD", "NZD", "SGD", "CHF", "CAD"]
+    al = [
+        "MYR",
+        "RUB",
+        "ZAR",
+        "KRW",
+        "AED",
+        "SAR",
+        "HUF",
+        "PLN",
+        "DKK",
+        "SEK",
+        "NOK",
+        "TRY",
+        "MXN",
+        "THB",
+    ]
+    is_inverse = False
+    if (currency[:3] in al) or (currency[4:] in bl):
+        is_inverse = True
+        currency = currency[4:] + "/" + currency[:3]
     url = "http://www.chinamoney.com.cn/ags/ms/cm-u-bk-ccpr/CcprHisNew?startDate={start_str}&endDate={end_str}&currency={currency}&pageNum=1&pageSize=300"
     if not end:
         end_obj = today_obj()
@@ -268,6 +336,8 @@ def get_rmb(start=None, end=None, prev=360, currency="USD/CNY"):
     df = pd.DataFrame(data)
     df = df[::-1]
     df["close"] = pd.to_numeric(df["close"])
+    if is_inverse:
+        df["close"] = 1 / df["close"]
     return df
 
 
@@ -329,7 +399,8 @@ selectedModule=PerformanceGraphView&selectedSubModule=Graph\
 &yearFlag={flag}YearFlag&indexId={code}".format(
         flag=flag, code=code
     )
-    df = pd.read_excel(url)
+    r = rget(url)
+    df = pd.read_excel(r.content)
     # print(df.iloc[:10])
     df = df.iloc[6:]
     df["close"] = df["Unnamed: " + col]
@@ -341,7 +412,7 @@ selectedModule=PerformanceGraphView&selectedSubModule=Graph\
 def get_historical_frombb(code, start=None, end=None, **kws):
     """
     https://www.bloomberg.com/ 数据源, 试验性支持。
-    似乎有很严格的 IP 封禁措施, 且最新数据更新滞后，似乎难以支持 T-1 净值预测。强烈建议从英为或雅虎能找到的标的，不要用彭博源，该 API 只能作为 last resort。
+    似乎有很严格的 IP 封禁措施, 且最新数据更新滞后，且国内会被 reset，似乎难以支持 T-1 净值预测。强烈建议从英为或雅虎能找到的标的，不要用彭博源，该 API 只能作为 last resort。
 
     :param code:
     :param start:
@@ -383,6 +454,46 @@ timeframe={years}&period=daily&volumePeriod=daily".format(
     return df
 
 
+def get_historical_fromft(code, start, end):
+    """
+    finance times 数据
+
+    :param code:
+    :param start:
+    :param end:
+    :return:
+    """
+    if not code.isdigit():
+        code = get_ft_id(code)
+    start = start.replace("/", "").replace("-", "")
+    end = end.replace("/", "").replace("-", "")
+    start = start[:4] + "/" + start[4:6] + "/" + start[6:]
+    end = end[:4] + "/" + end[4:6] + "/" + end[6:]
+    url = "https://markets.ft.com/data/equities/ajax/\
+get-historical-prices?startDate={start}&endDate={end}&symbol={code}".format(
+        code=code, start=start, end=end
+    )
+    r = rget_json(url, headers={"user-agent": "Mozilla/5.0"})
+    b = BeautifulSoup(r["html"], "lxml")
+    data = {"date": [], "open": [], "close": [], "high": [], "low": []}
+    for i, td in enumerate(b.findAll("td")):
+        if i % 6 == 0:
+            s = td.find("span").string.split(",")[1:]
+            s = ",".join(s)
+            data["date"].append(dt.datetime.strptime(s, " %B %d, %Y"))
+        elif i % 6 == 1:
+            data["open"].append(_float(td.string))
+        elif i % 6 == 2:
+            data["high"].append(_float(td.string))
+        elif i % 6 == 3:
+            data["low"].append(_float(td.string))
+        elif i % 6 == 4:
+            data["close"].append(_float(td.string))
+    df = pd.DataFrame(data)
+    df = df.iloc[::-1]
+    return df
+
+
 def get_historical_fromyh(code, start=None, end=None):
     """
     雅虎财经数据源，支持数据丰富，不限于美股。但存在部分历史数据确实 NAN 或者周末进入交易日的现象，可能数据需要进一步清洗和处理。
@@ -434,6 +545,19 @@ def get_historical_fromyh(code, start=None, end=None):
     return df
 
 
+@data_source("jq")
+def get_macro(table, start, end, datecol="stat_year"):
+    df = macro.run_query(
+        query(getattr(macro, table))
+        .filter(getattr(getattr(macro, table), datecol) >= start)
+        .filter(getattr(getattr(macro, table), datecol) <= end)
+        .order_by(getattr(getattr(macro, table), datecol))
+    )
+    df[datecol] = pd.to_datetime(df[datecol])
+    df["date"] = df[datecol]
+    return df
+
+
 def _get_daily(code, start=None, end=None, prev=365, _from=None, wrapper=True, **kws):
     """
     universal fetcher for daily historical data of literally everything has a value in market.
@@ -471,6 +595,10 @@ def _get_daily(code, start=None, end=None, prev=365, _from=None, wrapper=True, *
 
             15. 形如 YH-CSGOLD.SW 格式的数据，返回雅虎财经标的日线数据（最近十年）。代码来自标的网页 url：https://finance.yahoo.com/quote/CSGOLD.SW。
 
+            16. 形如 FT-22065529 格式的数据或 FT-INX:IOM，可以返回 financial times 的数据，推荐直接用后者。前者数字代码来源，打开浏览器 network 监视，切换图标时间轴时，会新增到 https://markets.ft.com/data/chartapi/series 的 XHR 请求，其 request payload 里的 [elements][symbol] 即为该指数对应数字。
+
+            17. 形如 mcy-MAC_AREA_UNEMPLOY 格式的数据，返回相应的宏观数据，需要聚宽数据源。mcy，mcq，mcm 代表年度，季度和月度的数据，code 为表名，可以参考 https://www.joinquant.com/help/api/help?name=macroData
+
     :param start: str. "20200101", "2020/01/01", "2020-01-01" are all legal. The starting date of daily data.
     :param end: str. format is the same as start. The ending date of daily data.
     :param prev: Optional[int], default 365. If start is not specified, start = end-prev.
@@ -494,9 +622,6 @@ def _get_daily(code, start=None, end=None, prev=365, _from=None, wrapper=True, *
             _from = "xueqiu"
         elif code.endswith("/CNY") or code.startswith("CNY/"):
             _from = "zjj"
-        elif len(code[1:].split("/")) == 2:
-            _from = "cninvesting"
-            code = get_investing_id(code)
         elif code.isdigit():
             _from = "cninvesting"
         elif code[0] in ["F", "M"] and code[1:].isdigit():
@@ -506,19 +631,21 @@ def _get_daily(code, start=None, end=None, prev=365, _from=None, wrapper=True, *
             code = code[2:]
         elif code.startswith("SP") and code[2:].split(".")[0].isdigit():
             _from = "SP"
-        elif len(code.split("-")) == 2 and len(code.split("-")[0]) <= 3:
+        elif len(code.split("-")) >= 2 and len(code.split("-")[0]) <= 3:
             # peb-000807.XSHG
             _from = code.split("-")[0]
-            code = code.split("-")[1]
+            code = "-".join(code.split("-")[1:])
+        elif len(code[1:].split("/")) == 2:
+            _from = "cninvesting"
+            code = get_investing_id(code)
         else:
             _from = "xueqiu"
 
     count = (today_obj() - start_obj).days + 1
     start_str = start_obj.strftime("%Y/%m/%d")
     end_str = end_obj.strftime("%Y/%m/%d")
-
     if _from in ["cninvesting", "investing", "default", "IN"]:
-        df = get_cninvesting(code, start_str, end_str)
+        df = get_historical_fromcninvesting(code, start_str, end_str)
         df = prettify(df)
     elif _from in ["xueqiu", "xq", "snowball", "XQ"]:
         df = get_historical_fromxq(code, count)
@@ -552,6 +679,23 @@ def _get_daily(code, start=None, end=None, prev=365, _from=None, wrapper=True, *
     elif _from == "YH":
         df = get_historical_fromyh(code, start=start, end=end)
 
+    elif _from == "FT":
+        df = get_historical_fromft(code, start=start, end=end)
+
+    elif _from == "INA":  # investing app
+        code = get_investing_id(code, app=True)
+        df = get_historical_fromcninvesting(code, start_str, end_str, app=True)
+        df = prettify(df)
+
+    elif _from == "mcy":
+        df = get_macro(code, start=start[:4], end=end[:4], datecol="stat_year")
+
+    elif _from == "mcq":
+        df = get_macro(code, start=start[:4], end=end[:4], datecol="stat_quarter")
+
+    elif _from == "mcm":
+        df = get_macro(code, start=start[:4], end=end[:4], datecol="stat_month")
+
     else:
         raise ParserFailure("no such data source: %s" % _from)
 
@@ -582,29 +726,72 @@ def get_xueqiu_rt(code, token="a664afb60c7036c7947578ac1a5860c4cfb6b3b5"):
     )
     n = r["data"]["quote"]["name"]
     q = r["data"]["quote"]["current"]
+    try:
+        q = _float(q)
+    except TypeError:  # 针对雪球实时在9点后开盘前可能出现其他情形的fixup， 效果待 check
+        # 现在的怀疑是在9am 到9:15 am, 雪球 API current 字段返回 Null
+        q = _float(r["data"]["quote"]["last_close"])
     q_ext = r["data"]["quote"].get("current_ext", None)
     percent = r["data"]["quote"]["percent"]
+    try:
+        percent = _float(percent)
+    except:
+        pass
     currency = r["data"]["quote"]["currency"]
+    market = r["data"]["market"]["region"]
     return {
         "name": n,
-        "current": _float(q),
-        "percent": _float(percent),
+        "current": q,
+        "percent": percent,
         "current_ext": _float(q_ext) if q_ext else None,
         "currency": currency,
+        "market": market,  # HK, US, CN
     }
 
 
-def get_cninvesting_rt(suburl):
-    url = "https://cn.investing.com"
+def get_cninvesting_rt(suburl, app=False):
+    trans = {
+        "瑞士": "CH",
+        "日本": "JP",
+        "美国": "US",
+        "香港": "HK",
+        "德国": "DE",
+        "英国": "UK",
+        "法国": "FR",
+        "中国": "CN",
+        "墨西哥": "MX",
+        "澳大利亚": "AU",
+    }
+    if not app:
+        url = "https://cn.investing.com"
+    else:
+        url = "https://cnappapi.investing.com"
     if not suburl.startswith("/"):
         url += "/"
     url += suburl
-    r = rget(
-        url,
-        headers={
+    if not app:
+        headers = {
             "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_4) AppleWebKit/537.36"
-        },
-    )
+        }
+    else:
+        headers = {
+            "Accept": "*/*",
+            "Accept-Encoding": "gzip",
+            "Accept-Language": "zh-cn",
+            "Cache-Control": "no-cache",
+            "Connection": "keep-alive",
+            "User-Agent": "Investing.China/0.0.3 CFNetwork/1121.2.2 Darwin/19.3.0",
+            "ccode": "CN",
+            #'ccode_time': '1585551041.986028',
+            "x-app-ver": "117",
+            "x-meta-ver": "14",
+            "x-os": "ios",
+            "x-uuid": str(uuid4()),
+            "Host": "cn.investing.com",
+            "X-Requested-With": "XMLHttpRequest",
+        }
+
+    r = rget(url, headers=headers,)
     s = BeautifulSoup(r.text, "lxml")
     last_last = s.find("span", id="last_last")
     q = _float(last_last.string)
@@ -627,12 +814,18 @@ def get_cninvesting_rt(suburl):
         q_ext = _float(panhou.find("span").string)
     else:
         q_ext = None
+    market = None
+    for span in s.findAll("span", class_="elp"):
+        if span.find("a") and span.find("a")["href"].startswith("/markets"):
+            market = span.string
+    market = trans.get(market, market)
     return {
         "name": name,
         "current": q,
         "current_ext": q_ext,
         "currency": currency,
         "percent": percent,
+        "market": market,
     }
 
 
@@ -641,6 +834,8 @@ def get_rt_from_sina(code):
         code.startswith("SH") or code.startswith("SZ") or code.startswith("HK")
     ) and code[2:].isdigit():
         tinycode = code[:2].lower() + code[2:]
+        if code.startswith("HK"):  # 港股额外要求实时
+            tinycode = "rt_" + tinycode
     else:  # 美股
         tinycode = "gb_"
         if code.startswith("."):
@@ -653,21 +848,83 @@ def get_rt_from_sina(code):
     if (
         code.startswith("SH") or code.startswith("SZ") or code.startswith("HK")
     ) and code[2:].isdigit():
-        d["current"] = float(l[3])
-        d["percent"] = round((float(l[3]) / float(l[2]) - 1) * 100, 2)
-        d["current_ext"] = None
         if code.startswith("HK"):
+            d["current"] = float(l[9])  # 英文股票名称占位
             d["currency"] = "HKD"
+            d["percent"] = round(float(l[8]), 2)
+            d["market"] = "HK"
         else:
+            d["current"] = float(l[3])
             d["currency"] = "CNY"
+            d["percent"] = round((float(l[3]) / float(l[2]) - 1) * 100, 2)
+            d["market"] = "CN"
+        d["current_ext"] = None
+
     else:
         d["currency"] = "USD"
         d["current"] = float(l[1])
         d["percent"] = float(l[2])
         d["current_ext"] = None
+        d["market"] = "US"
     return d
 
 
+@lru_cache(maxsize=512)
+def get_ft_id(code):
+    url = "https://markets.ft.com/data/indices/tearsheet/summary?s={code}".format(
+        code=code
+    )
+    r = rget(url)
+    b = BeautifulSoup(r.text, "lxml")
+    return eval(
+        b.find("section", class_="mod-tearsheet-add-to-watchlist")["data-mod-config"]
+    )["xid"]
+
+
+def get_rt_from_ft(code):
+    r = rget(
+        "https://markets.ft.com/data/indices/tearsheet/summary?s={code}".format(
+            code=code
+        )
+    )
+    b = BeautifulSoup(r.text, "lxml")
+    d = {}
+    d["name"] = b.find("h1").string
+    d["current"] = _float(b.find("span", class_="mod-ui-data-list__value").string)
+    d["percent"] = _float(
+        b.findAll("span", class_="mod-ui-data-list__value")[1]
+        .find("span")
+        .contents[1]
+        .split("/")[1][:-1]
+    )
+    d["current_ext"] = None
+    d["market"] = None
+    d["currency"] = b.find("span", class_="mod-ui-data-list__label").string.split("(")[
+        1
+    ][:-1]
+    return d
+
+
+def get_newest_netvalue(code):
+    """
+    防止天天基金总量 API 最新净值更新不及时，获取基金最新公布净值及对应日期
+
+    :param code: six digits string for fund.
+    :return: netvalue, %Y-%m-%d
+    """
+    code = code[1:]
+    r = rget("http://fund.eastmoney.com/{code}.html".format(code=code))
+    s = BeautifulSoup(r.text, "lxml")
+    return (
+        float(
+            s.findAll("dd", class_="dataNums")[1]
+            .find("span", class_="ui-font-large")
+            .string
+        ),
+        str(s.findAll("dt")[1]).split("(")[1].split(")")[0][7:],
+    )
+
+
 def get_rt(code, _from=None, double_check=False, double_check_threhold=0.005):
     """
     universal fetcher for realtime price of literally everything.
@@ -678,27 +935,51 @@ def get_rt(code, _from=None, double_check=False, double_check_threhold=0.005):
     :param double_check: Optional[bool], default False. 如果设为 True，只适用于 A 股，美股，港股实时行情，会通过至少两个不同的数据源交叉验证，确保正确。
             适用于需要自动交易等情形，防止实时数据异常。
     :return: Dict[str, Any].
-        包括 "name", "current", "percent" 三个必有项和 "current_ext"（盘后价格）, "currency" （计价货币）两个值可能为 ``None`` 的选项。
+        包括 "name", "current", "percent" 三个必有项和 "current_ext"（盘后价格）, "currency" （计价货币）， "market" (发行市场)可能为 ``None`` 的选项。
     """
+    # 对于一些标的，get_rt 的主任务可能不是 current 价格，而是去拿 market currency 这些元数据
+    # 现在用的新浪实时数据源延迟严重， double check 并不靠谱，港股数据似乎有15分钟延迟（已解决）
+    # 雪球实时和新浪实时在9：00之后一段时间可能都有问题
     if not _from:
-        if len(code.split("/")) > 1:
-            _from = "investing"
-        # elif code.startswith("HK") and code[2:].isdigit():
+        # if code.startswith("HK") and code[2:].isdigit():
         #     _from = "xueqiu"
-        else:  # 默认不启用新浪实时，只做双重验证
+        if len(code.split("-")) >= 2 and len(code.split("-")[0]) <= 3:
+            _from = code.split("-")[0]
+            code = "-".join(code.split("-")[1:])
+        elif len(code.split("/")) > 1:
+            _from = "investing"
+        else:  # 默认不启用新浪实时，只做双重验证备份
             _from = "xueqiu"
     if _from in ["cninvesting", "investing"]:
-        return get_cninvesting_rt(code)
-    elif double_check:
+        try:
+            return get_cninvesting_rt(code)
+        except Exception as e:
+            logger.warning(
+                "Fails due to %s, now trying app source of investing.com" % e.args[0]
+            )
+            return get_cninvesting_rt(code, app=True)
+    elif double_check and _from in ["xueqiu", "sina"]:
         r1 = get_xueqiu_rt(code, token=get_token())
         r2 = get_rt_from_sina(code)
         if abs(r1["current"] / r2["current"] - 1) > double_check_threhold:
             raise DataPossiblyWrong("realtime data unmatch for %s" % code)
         return r1
     elif _from in ["xueqiu", "xq", "snowball"]:
-        return get_xueqiu_rt(code, token=get_token())
+        try:
+            return get_xueqiu_rt(code, token=get_token())
+        except Exception as e:  # 默认雪球实时引入备份机制
+            logging.warning(
+                "Fails due to %s, now trying backup data source from sina" % e.args[0]
+            )
+            return get_rt_from_sina(code)
     elif _from in ["sina", "sn", "xinlang"]:
         return get_rt_from_sina(code)
+    elif _from in ["FT", "ft"]:
+        return get_rt_from_ft(code)
+    elif _from in ["INA"]:  # investing app
+        return get_cninvesting_rt(code, app=True)
+    else:
+        raise ParserFailure("unrecoginzed _from for %s" % _from)
 
 
 get_realtime = get_rt
@@ -905,7 +1186,7 @@ def cachedio(**ioconf):
                         if precached:
                             kws["start"] = precached.replace("/", "").replace("-", "")
                             kws["end"] = (today_obj() - dt.timedelta(days=1)).strftime(
-                                "%Y-%m-%d"
+                                "%Y%m%d"
                             )
                         is_changed = True
                         df0 = f(*args, **kws)
@@ -920,7 +1201,7 @@ def cachedio(**ioconf):
                     d = getattr(thismodule, "cached_dict")
                     d[key] = df0
 
-            if df0 is not None:
+            if df0 is not None and len(df0) > 0:
                 df0 = df0[df0["date"] <= end_str]
                 df0 = df0[df0["date"] >= start_str]
 
@@ -959,7 +1240,7 @@ def _get_index_weight_range(code, start, end):
 
             df["date"] = pd.to_datetime(df["date"])
             return df
-        print("call", d, " ", code)
+        logger.debug("fetch index weight on %s for %s" % (d, code))
         df0 = get_index_weights(index_id=code, date=d.strftime("%Y-%m-%d"))
         df0["code"] = df0.index
         df = df.append(df0, ignore_index=True)
@@ -981,7 +1262,7 @@ def _get_peb_range(code, start, end):  # 盈利，净资产，总市值
     data = {"date": [], "pe": [], "pb": []}
     for d in pd.date_range(start=start, end=end, freq="W-FRI"):
         data["date"].append(d)
-        print("compute pe pb on %s" % d)
+        logger.debug("compute pe pb on %s" % d)
         r = get_peb(code, date=d.strftime("%Y-%m-%d"))
         data["pe"].append(r["pe"])
         data["pb"].append(r["pb"])
@@ -1118,3 +1399,47 @@ def _inverse_convert_code(code):
         return code[2:] + ".XSHG"
     elif code.startswith("SZ"):
         return code[2:] + ".XSHE"
+
+
+def get_bar(code, prev=120, interval=3600):
+    """
+    get bar data beyond daily bar
+
+    :param code: str. investing id or url
+    :param prev: int, data points from now, max might be around 500, if exceed, only None is returnd
+    :param interval: default 3600. optional 60, 300, 900, 1800, 18000, 86400, "week", "month"
+    :return: pd.DataFrame or None if prev and interval unmatch the API
+    """
+    if interval == "day":
+        interval = 86400
+    elif interval == "hour":
+        interval = 3600
+    elif interval == "minute":
+        interval = 60
+    if len(code.split("/")) == 2:
+        code = get_investing_id(code)
+    r = rget_json(
+        "https://cn.investing.com/common/modules/js_instrument_chart/api/data.php?pair_id={code}&pair_id_for_news={code}\
+&chart_type=area&pair_interval={interval}&candle_count={prev}&events=yes&volume_series=yes&period=".format(
+            code=code, prev=str(prev), interval=str(interval)
+        ),
+        headers={
+            "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_4)\
+                    AppleWebKit/537.36 (KHTML, like Gecko)",
+            "Host": "cn.investing.com",
+            "Referer": "https://cn.investing.com/commodities/",
+            "Sec-Fetch-Dest": "empty",
+            "Sec-Fetch-Mode": "cors",
+            "Sec-Fetch-Site": "same-origin",
+            "X-Requested-With": "XMLHttpRequest",
+        },
+    )
+    if not r:
+        return  # None
+    tz_bj = dt.timezone(dt.timedelta(hours=8))
+    df = pd.DataFrame(r["candles"], columns=["date", "close", "0", "1"])
+    df = df.drop(["0", "1"], axis=1)
+    df["date"] = df["date"].apply(
+        lambda t: dt.datetime.fromtimestamp(t / 1000, tz=tz_bj).replace(tzinfo=None)
+    )
+    return df
